@@ -11,7 +11,7 @@ serializable properties.
 from __future__ import absolute_import
 
 import logging
-logger = logging.getLogger(__name__)
+log = logging.getLogger(__name__)
 
 import difflib
 import inspect
@@ -28,16 +28,6 @@ from .property.containers import PropertyValueContainer
 from .property.descriptor_factory import PropertyDescriptorFactory
 from .property.override import Override
 
-IPython = import_optional('IPython')
-
-if IPython:
-    from IPython.lib.pretty import RepresentationPrinter
-
-    class _BokehPrettyPrinter(RepresentationPrinter):
-        def __init__(self, output, verbose=False, max_width=79, newline='\n'):
-            super(_BokehPrettyPrinter, self).__init__(output, verbose, max_width, newline)
-            self.type_pprinters[HasProps] = lambda obj, p, cycle: obj._repr_pretty(p, cycle)
-
 _ABSTRACT_ADMONITION = '''
     .. note::
         This is an abstract base class used to help organize the hierarchy of Bokeh
@@ -250,7 +240,6 @@ class HasProps(with_metaclass(MetaHasProps, object)):
         a few situations:
 
         * short circuit all property machinery for ``_private`` attributes
-        * handle setting ``__deprecated_attributes__``
         * suggest similar attribute names on attribute errors
 
         Args:
@@ -268,9 +257,9 @@ class HasProps(with_metaclass(MetaHasProps, object)):
             return
 
         props = sorted(self.properties())
-        deprecated = getattr(self, '__deprecated_attributes__', [])
+        descriptor = getattr(self.__class__, name, None)
 
-        if name in props or name in deprecated:
+        if name in props or (descriptor is not None and descriptor.fset is not None):
             super(HasProps, self).__setattr__(name, value)
         else:
             matches, text = difflib.get_close_matches(name.lower(), props), "similar"
@@ -336,11 +325,11 @@ class HasProps(with_metaclass(MetaHasProps, object)):
 
         '''
         if name in self.properties():
-            #logger.debug("Patching attribute %s of %r", attr, patched_obj)
+            log.trace("Patching attribute %r of %r with %r", name, self, json)
             descriptor = self.lookup(name)
             descriptor.set_from_json(self, json, models, setter)
         else:
-            logger.warn("JSON had attr %r on obj %r, which is a client-only or invalid attribute that shouldn't have been sent", name, self)
+            log.warn("JSON had attr %r on obj %r, which is a client-only or invalid attribute that shouldn't have been sent", name, self)
 
     def update(self, **kwargs):
         ''' Updates the object's properties from the given keyword arguments.
@@ -656,6 +645,16 @@ class HasProps(with_metaclass(MetaHasProps, object)):
             ValueError, if ``IPython`` cannot be imported
 
         '''
+        IPython = import_optional('IPython')
+
+        if IPython:
+            from IPython.lib.pretty import RepresentationPrinter
+
+        class _BokehPrettyPrinter(RepresentationPrinter):
+            def __init__(self, output, verbose=False, max_width=79, newline='\n'):
+                super(_BokehPrettyPrinter, self).__init__(output, verbose, max_width, newline)
+                self.type_pprinters[HasProps] = lambda obj, p, cycle: obj._repr_pretty(p, cycle)
+
         if not IPython:
             cls = self.__class__
             raise RuntimeError("%s.%s.pretty() requires IPython" % (cls.__module__, cls.__name__))
