@@ -26,7 +26,7 @@ from ..core.enums import Anchor, Dimension, Dimensions, Location, TooltipFieldFo
 from ..core.has_props import abstract
 from ..core.properties import (
     Auto, Bool, Color, Dict, Either, Enum, Float, Percent, Instance, List,
-    Override, Seq, String, Tuple
+    Seq, String, Tuple
 )
 from ..model import Model
 from ..util.deprecation import deprecated
@@ -34,7 +34,7 @@ from ..util.deprecation import deprecated
 from .annotations import BoxAnnotation, PolyAnnotation
 from .callbacks import Callback
 from .renderers import Renderer
-from .layouts import Box, LayoutDOM
+from .layouts import LayoutDOM
 
 
 @abstract
@@ -43,8 +43,6 @@ class Tool(Model):
 
     '''
 
-    __deprecated_attributes__ = ["plot"]
-
     @property
     def plot(self):
         deprecated("Tool.plot property is no longer needed, and any use deprecated. In the future, accessing Tool.plot will result in an AttributeError")
@@ -96,7 +94,7 @@ class Inspection(Tool):
     """)
 
 @abstract
-class ToolbarBase(LayoutDOM):
+class ToolbarBase(Model):
     ''' A base class for different toolbars.
 
     '''
@@ -110,11 +108,6 @@ class ToolbarBase(LayoutDOM):
     A list of tools to add to the plot.
     """)
 
-    # This is an odd case. The sizing is custom handled. In the future we will
-    # probably set it as `stretch_width` or `stretch_height` depending on its
-    # orientation.
-    sizing_mode = Override(default=None)
-
 
 class Toolbar(ToolbarBase):
     ''' Collect tools to display for a single plot.
@@ -138,35 +131,20 @@ class Toolbar(ToolbarBase):
     Specify a tap/click tool to be active when the plot is displayed.
     """)
 
+class ProxyToolbar(ToolbarBase):
+    ''' A toolbar that allow to merge and proxy tools of toolbars in multiple plots. '''
 
-class ToolbarBox(Box):
+class ToolbarBox(LayoutDOM):
     ''' A layoutable toolbar that can accept the tools of multiple plots, and
     can merge the tools into a single button for convenience.
 
     '''
-    def _check_empty_layout(self):
-        # Overriding the children check from Box. As toolbarbox's children
-        # are normally set JS side.
-        return None
 
-    toolbar_location = Enum(Location, default='right', help="""
-        Should the toolbar be presented as if it was stuck to the `above`, `right`, `left`, `below`
-        edge of a plot. Default is `right`.
-    """)
-
-    tools = List(Instance(Tool), help="""
-    A list of tools to add to the plot.
-    """)
-
-    merge_tools = Bool(default=True, help="""
-        Merge all the tools together so there is one tool to control all the plots.
-    """)
-
-    logo = Enum("normal", "grey", help="""
-    What version of the Bokeh logo to display on the toolbar. If
-    set to None, no logo will be displayed.
+    toolbar = Instance(ToolbarBase, help="""
+    A toolbar associated with a plot which holds all its tools.
     """)
 
+    toolbar_location = Enum(Location, default="right")
 
 class PanTool(Drag):
     ''' *toolbar icon*: |pan_icon|
@@ -673,8 +651,13 @@ class HoverTool(Inspection):
     off by setting ``tooltips=None``.
 
     .. warning::
+        When supplying a callback or custom template, the explicit intent
+        of this Bokeh Model is to embed *raw HTML and  JavaScript code* for
+        a browser to execute. If any part of the code is derived from untrusted
+        user inputs, then you must take appropriate care to sanitize the user
+        input prior to passing to Bokeh.
 
-        Hover tool does not currently work with the following glyphs:
+    Hover tool does not currently work with the following glyphs:
 
         .. hlist::
             :columns: 3
@@ -719,7 +702,7 @@ class HoverTool(Inspection):
             default=[
                 ("index","$index"),
                 ("data (x, y)","($x, $y)"),
-                ("canvas (x, y)","($sx, $sy)"),
+                ("screen (x, y)","($sx, $sy)"),
             ], help="""
     The (name, field) pairs describing what the hover tool should
     display when there is a hit.
