@@ -406,6 +406,8 @@ _known_tools = {
     "pan": lambda: PanTool(dimensions='both'),
     "xpan": lambda: PanTool(dimensions='width'),
     "ypan": lambda: PanTool(dimensions='height'),
+    "xwheel_pan": lambda: WheelPanTool(dimension="width"),
+    "ywheel_pan": lambda: WheelPanTool(dimension="height"),
     "wheel_zoom": lambda: WheelZoomTool(dimensions='both'),
     "xwheel_zoom": lambda: WheelZoomTool(dimensions='width'),
     "ywheel_zoom": lambda: WheelZoomTool(dimensions='height'),
@@ -415,8 +417,6 @@ _known_tools = {
     "zoom_out": lambda: ZoomOutTool(dimensions='both'),
     "xzoom_out": lambda: ZoomOutTool(dimensions='width'),
     "yzoom_out": lambda: ZoomOutTool(dimensions='height'),
-    "xwheel_pan": lambda: WheelPanTool(dimension="width"),
-    "ywheel_pan": lambda: WheelPanTool(dimension="height"),
     "click": lambda: TapTool(behavior="inspect"),
     "tap": lambda: TapTool(),
     "crosshair": lambda: CrosshairTool(),
@@ -431,7 +431,7 @@ _known_tools = {
     "hover": lambda: HoverTool(tooltips=[
         ("index", "$index"),
         ("data (x, y)", "($x, $y)"),
-        ("canvas (x, y)", "($sx, $sy)"),
+        ("screen (x, y)", "($sx, $sy)"),
     ]),
     "save": lambda: SaveTool(),
     "previewsave": "save",
